/*
AIR BRAKE
CTR-style air-braking
*/

local cv_airbrake = CV_RegisterVar({
    name = "wa_airbrake",
    defaultvalue = "On",
    flags = CV_NETVAR,
    PossibleValue = CV_OnOff
})

freeslot("MT_TETHERDUSTTRAIL", "SPR_THDT", "S_TETHDUST1", "S_TETHDUST2", "S_FADEDUST1", "S_FADEDUST2");
states[S_TETHDUST1] = {SPR_THDT, B|FF_FULLBRIGHT, 1, nil, 0, 0, S_TETHDUST2}
states[S_TETHDUST2] = {SPR_THDT, A|FF_FULLBRIGHT|FF_ANIMATE, 4, nil, 3, 1, S_NULL}

mobjinfo[MT_TETHERDUSTTRAIL] = {
		doomednum = -1,
		spawnstate = S_TETHDUST1,    // spawnstate
		spawnhealth = 1,              // spawnhealth
		reactiontime = 8,              // reactiontime
		speed = 8,              // speed
		radius = 14*FRACUNIT,    // radius
		height = 14*FRACUNIT,    // height
		dispoffset = 0,              // display offset
		mass = 100,            // mass
		flags = MF_NOBLOCKMAP|MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOGRAVITY|MF_SCENERY|MF_SLIDEME|MF_DONTENCOREMAP, // flags
		raisestate = S_NULL          // raisestate
}

// lua'd this from the source
// Adds gravity flipping to an object relative to its master and shifts the z coordinate accordingly.
rawset(_G, "K_FlipFromObject", function(mo, master)
	mo.eflags = (mo.eflags & ~MFE_VERTICALFLIP)|(master.eflags & MFE_VERTICALFLIP);
	mo.flags2 = (mo.flags2 & ~MF2_OBJECTFLIP)|(master.flags2 & MF2_OBJECTFLIP);

	if (mo.eflags & MFE_VERTICALFLIP)
		mo.z = ($ + master.height - FixedMul(master.scale, mo.height));
	end
end)

rawset(_G, "K_SpawnAirBrakeDust", function(player, colorize) -- adjustable boost trail
	local newx; --fixed_t
	local newy; --fixed_t
	local ground; --fixed_t
	local flame; -- obj
	local travelangle; -- angle

	if (not (player.mo and player.mo.valid)) then return end;

	if ((G_BattleGametype() and player.kartstuff[k_bumper] <= 0 and player.kartstuff[k_comebacktimer])) // show sneaker boosts during a hyuu
		return;
    end

	if (player.mo.eflags & MFE_VERTICALFLIP)
		ground = player.mo.ceilingz - FixedMul(mobjinfo[MT_SNEAKERTRAIL].height, player.mo.scale);
	else
		ground = player.mo.floorz;
    end

	if (player.kartstuff[k_drift] != 0)
		travelangle = player.mo.angle;
	else
		travelangle = R_PointToAngle2(0, 0, player.rmomx, player.rmomy);
	end
	
	for j = 0, min(3, max(0, ((player.brakeholdtime) and (( ((fade) and 15 or player.brakeholdtime) ) / (TICRATE/4)) or 0) ))
		for i = 0, 2
			local tx = P_ReturnThrustX(player.mo, travelangle, FixedMul((-12*FRACUNIT)*(j), player.mo.scale))
			local ty = P_ReturnThrustY(player.mo, travelangle, FixedMul((-12*FRACUNIT)*(j), player.mo.scale));
			local tz = player.mo.momz * j;
			
			newx = player.mo.x + P_ReturnThrustX(player.mo, travelangle + ((i&1) and -1 or 1)*ANGLE_135, FixedMul((24*FRACUNIT), player.mo.scale)) + tx;
			newy = player.mo.y + P_ReturnThrustY(player.mo, travelangle + ((i&1) and -1 or 1)*ANGLE_135, FixedMul((24*FRACUNIT), player.mo.scale)) + ty;
			if (player.mo.standingslope)
				ground = P_GetZAt(player.mo.standingslope, newx, newy);
				if (player.mo.eflags & MFE_VERTICALFLIP)
					ground = ($ - FixedMul(mobjinfo[MT_SNEAKERTRAIL].height, player.mo.scale));
				end
			end
			flame = P_SpawnMobj(newx, newy, player.mo.z-(tz/4), MT_TETHERDUSTTRAIL);
			
			if (colorize)
				local setcolor = ColorOpposite(player.skincolor);
				flame.color = setcolor;
				flame.colorized = true;
			end

			flame.target = player.mo;
			flame.angle = travelangle;
			flame.sloperoll = player.mo.sloperoll;
			flame.fuse = TICRATE*2;
			flame.destscale = player.mo.scale;
			P_SetScale(flame, player.mo.scale);
			// not K_MatchGenericExtraFlags so that a stolen sneaker can be seen
			K_FlipFromObject(flame, player.mo);

			flame.momx = 8;

			if (not (flame and flame.valid)) then continue end;
		end
	end
end)

local ANG1 = ANG1;
local ANGTenth = ANG1/10;
local ORIG_FRICTION = 62914;
local brakedelaytime = TICRATE/3;
local TICRATE_h = TICRATE/2;

local function midairBrake(p)
	if (not cv_airbrake.value) then return; end
	if (not (p.mo and p.mo.valid)) then return; end
	if (p.grinding) or (p.railjumped) then return; end --Air brakes don't like rails much
	if (p.mo.paragliding) then return; end --Air brakes don't like gliders much
	
    local pks = p.kartstuff;
	local doBrakeDelay = (p.brakedelay)
	local pflip;
	local cantairbrake = (pks[k_spinouttimer] or p.playerstate == PST_DEAD or pks[k_respawn])
	local noaccel = false;	
		
	if (not (p.cmd.buttons & BT_ACCELERATE))	
		noaccel = true;	
		//print("no accel");	
	end
	
	if (pks[k_spinouttimer]) then return; end
	
	
	

	
	pflip = (p.mo.eflags & MFE_VERTICALFLIP);
	
	if (not P_IsObjectOnGround(p.mo)) 
		if (not p.brakedelayset)
			p.brakedelay = brakedelaytime;
			p.brakedelayset = true;
		end
		
		if ((p.cmd.buttons & BT_BRAKE) and (not doBrakeDelay) and (not cantairbrake))
			/*if (p.ab_zdist == nil)
				S_StartSound(p.mo, sfx_ddash);
				p.ab_zdist = R_PointToDist2(0, p.mo.z, 0, (pflip and p.mo.ceilingz or p.mo.floorz))/(((p.mo.scale) and p.mo.scale or mapobjectscale)*6);
				if (pks[k_sneakertimer] and (not p.airbrakesneakertime)) then p.airbrakesneakertime = pks[k_sneakertimer]; end
				
				if (p.stackedpanels)
					p.pstackmem = p.stackedpanels;
				end
					
				if (p.stackedsneakers)
					p.istackmem = p.stackedsneakers;
				end
			end*/
			
			p.brakeholdtime = ((p.brakeholdtime) and ($ + 1) or 1);
			
			//SpawnReverseFastLines(p, p.mo.scale, ColorOpposite(p.skincolor), true);
			K_SpawnAirBrakeDust(p, ((p.airbrakesneakertime) and true or false));
			
			//p.mo.momx = FixedMul($, ORIG_FRICTION);	
			//p.mo.momy = FixedMul($, ORIG_FRICTION);	
				
			// you instead get heavier	
			p.mo.momz = $ + ((p.mo.eflags & MFE_VERTICALFLIP) and (gravity) or (-gravity));
		end
	else
		p.brakedelayset = false;
		
		p.ab_speedmem = p.speed;
		/*if (p.ab_zdist)
			S_StartSound(p.mo, sfx_cdfm62);
			if (pks[k_sneakertimer] or p.airbrakesneakertime)
				local airboost = min(85, pks[k_sneakertimer] + ((p.airbrakesneakertime ~= nil) and max(1, (3*p.airbrakesneakertime/4)) or 0) + (p.ab_zdist/2));
				
				if (p.panelsneakertimer ~= nil) // slipstream?
					if (p.pstackmem or p.istackmem)
						local ps = ((p.pstackmem) and p.pstackmem or 0);
						local is = ((p.istackmem) and p.istackmem or 0);
						
						p.stackedpanels = ps + is;
						p.pstackmem = nil;
						p.istackmem = nil;
					end
					
					p.naturalpanelsneakertimer = airboost;
					p.panelsneakertimer = airboost;
				end
				pks[k_sneakertimer] = airboost;
			end
			
			if (p.airbrakesneakertime)
				local overlay = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_BOOSTFLAME);
				overlay.target = p.mo;
				overlay.destscale = p.mo.scale;
				P_SetScale(overlay, overlay.destscale);
				K_FlipFromObject(overlay, p.mo);
			end

			
			p.airbrakesneakertime = nil;
			
			pks[k_startboost] = min(90, ((p.ab_zdist*2)/3)*max(1, min(2, (p.brakeholdtime/TICRATE))) ); -- longer you brake, better the brakeboost
		end*/
		
		p.airbrakesneakertime = nil;
		
		p.brakeholdtime = 0;
		p.ab_zdist = nil;
		p.ab_speedmem = p.speed;
	end
	
	if (p.brakedelay)
		p.brakedelay = $ - 1;
	end
end

local function airbrakepspawn(p)
	// clear out players when they spawn/respawn
	p.ab_zdist = nil;
	p.airbrakesneakertime = nil;
	p.pstackmem = nil;
	p.istackmem = nil;
	p.brakeholdtime = 0;
end

addHook("PlayerThink", midairBrake)
addHook("PlayerSpawn", airbrakepspawn)